package com.asahi.springframework.beans.factory.support;

import com.asahi.springframework.beans.BeansException;
import com.asahi.springframework.beans.factory.BeanFactory;
import com.asahi.springframework.beans.factory.config.BeanDefinition;
import com.asahi.springframework.beans.factory.config.DefaultSingletonBeanRegistry;

/**
 * Description： 定义抽象Bean工厂
 *
 * @ClassName AbstractBeanFactory
 * @Author Aashi
 * @Date 2022/6/14 13:47
 * @Version 1.0
 */
public abstract class AbstractBeanFactory extends DefaultSingletonBeanRegistry implements BeanFactory {

    @Override
    public Object getBean(String name) throws BeansException {
        Object bean = getSingleton(name);
        if (bean != null) {
            return bean;
        }
        BeanDefinition beanDefinition = getBeanDefinition(name);
        return createBean(name, beanDefinition);
    }

    protected abstract BeanDefinition getBeanDefinition(String beanName) throws BeansException;

    protected abstract Object createBean(String beanName, BeanDefinition beanDefinition) throws BeansException;

}
